package tech.graphits.inserter;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.stream.Collectors;

import org.junit.Assert;
import org.junit.Test;
import org.neo4j.graphdb.GraphDatabaseService;
import org.neo4j.graphdb.Label;
import org.neo4j.graphdb.Transaction;
import org.neo4j.graphdb.factory.GraphDatabaseFactory;

/**
 * Test simple qui lance l'insertion et vrifie le nombre de donnes dans le graphe
 * @author sroussy
 *
 */
public class LivreInserterTest {
	
	public static final String GRAPH_DB_PATH = "catalogue.graphdb";
	
	 @Test
	    public void testBatchInserter (){
		  GraphDatabaseService graphDb =null;
	        try {
	        	final LivreInserter inserter = new LivreInserter(GRAPH_DB_PATH);
	        	inserter.execute();
	        	
	        	graphDb = new GraphDatabaseFactory().newEmbeddedDatabase(new File(GRAPH_DB_PATH));
	        	
	        	try(Transaction tx = graphDb.beginTx()){
		        	final long nombreLivres = graphDb.findNodes(Label.label("Livre")).stream().collect(Collectors.counting());
		        	Assert.assertEquals(2L, nombreLivres);
		        	
		        	final long nombreAuteurs = graphDb.findNodes(Label.label("Auteur")).stream().collect(Collectors.counting());
		        	Assert.assertEquals(3L, nombreAuteurs);
		        	
		        	
		        	final long nombreRelations = graphDb.getAllRelationships().stream().collect(Collectors.counting());
		        	Assert.assertEquals(5L, nombreRelations);
	        	}
	        	
	        	
	        	graphDb.shutdown();
	        	
	        } catch (IOException e) {
	          
	        	Assert.fail(e.getMessage());
	            e.printStackTrace();
	        }
	        finally{
	        	if (graphDb != null) graphDb.shutdown();
	        }
	    }
}
