package tech.graphits.catalogue;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;



import java.util.List;
import java.util.Map;

import org.junit.Assert;
import org.junit.Test;

import com.fasterxml.jackson.databind.ObjectMapper;
/**
 * Tests basés sur la combinaison JDBC et BOLT
 * @author sroussy
 *
 */
public class JdbcCatalogueTest {

	private static final ObjectMapper MAPPER = new ObjectMapper();
	
	// A remplacer par votre Url
	private static final String JDBC_BOLT_URL ="jdbc:neo4j:bolt://localhost";
	
	// A remplacer par votre utilisateur
	private static final String USER ="neo4j";
	
	// A remplacer par votre mot de passe
	private static final String PASSWORD ="root";
	
	/**
	 * Test simple de connexion JDBC 
	 */
	@Test
	public void testJdbcConnection (){		
		
		
		try (Connection con = DriverManager.getConnection(JDBC_BOLT_URL, USER, PASSWORD)) {
			
		
			
		} catch (SQLException e) {			
			e.printStackTrace();
			
			Assert.fail(e.getMessage());
		}

		
		
	}
	
	/**
	 * Test de requêtes JDBC : créer puis retrouver une personne
	 */
	@Test
	public void testJdbcQuery (){
		
		
		final String writeQuery = "MERGE (p:Personne {nom: {1}})";
		
		
		try (Connection con = DriverManager.getConnection(JDBC_BOLT_URL, USER, PASSWORD)) {
			
			PreparedStatement pstmt = con.prepareStatement(writeQuery);
			pstmt.setString(1, "Patricia");
			pstmt.executeUpdate();
			
		} catch (SQLException e) {			
			e.printStackTrace();
			
			Assert.fail(e.getMessage());
		}
		
		final String readQuery ="MATCH (p:Personne {nom:{1}}) RETURN p.nom AS nom";
		try (Connection con = DriverManager.getConnection(JDBC_BOLT_URL, USER, PASSWORD)) {
			
			PreparedStatement pstmt = con.prepareStatement(readQuery);
			pstmt.setString(1, "Patricia");
			
			try (ResultSet rs = pstmt.executeQuery()) {
	            while (rs.next()) {
	                System.out.println("Personne: "+rs.getString("nom"));
	            }
	        }
			catch (SQLException err) {			
				err.printStackTrace();
				Assert.fail(err.getMessage());
			}

			
			
		} catch (SQLException e) {			
			e.printStackTrace();
			
			Assert.fail(e.getMessage());
		}

		
		
	}
	
	/**
	 * Test de requêtes JDBC : créer puis retrouver une personne avec mapping de type Map
	 */
	@Test
	public void testJdbcObject (){
		
		
		final String writeQuery = "MERGE (p:Personne {nom: {1}})";
		
		
		try (Connection con = DriverManager.getConnection(JDBC_BOLT_URL, USER, PASSWORD)) {
			
			PreparedStatement pstmt = con.prepareStatement(writeQuery);
			pstmt.setString(1, "Patricia");
			pstmt.executeUpdate();
			
		} catch (SQLException e) {			
			e.printStackTrace();
			
			Assert.fail(e.getMessage());
		}
		
		final String readQuery ="MATCH (p:Personne {nom:{1}}) RETURN p";
		try (Connection con = DriverManager.getConnection(JDBC_BOLT_URL, USER, PASSWORD)) {
			
			PreparedStatement pstmt = con.prepareStatement(readQuery);
			pstmt.setString(1, "Patricia");
			
			try (ResultSet rs = pstmt.executeQuery()) {
	            while (rs.next()) { 
	            	Map<String,Object> noeud = (Map<String,Object>) rs.getObject("p"); 
	                System.out.println("Personne (Map) : "+noeud);
	            }
	        }
			catch (SQLException err) {			
				err.printStackTrace();
				Assert.fail(err.getMessage());
			}

			
			
		} catch (SQLException e) {			
			e.printStackTrace();
			
			Assert.fail(e.getMessage());
		}

		
		
	}
	
	/**
	 * Test de requêtes JDBC : créer puis retrouver une personne avec mapping de type objet Personne
	 */
	@Test
	public void testJdbcPersonne (){
		
		
		final String writeQuery = "MERGE (p:Personne {nom: {1}})";
		
		
		try (Connection con = DriverManager.getConnection(JDBC_BOLT_URL, USER, PASSWORD)) {
			
			PreparedStatement pstmt = con.prepareStatement(writeQuery);
			pstmt.setString(1, "Patricia");
			pstmt.executeUpdate();
			
		} catch (SQLException e) {			
			e.printStackTrace();
			
			Assert.fail(e.getMessage());
		}
		
		final String readQuery ="MATCH (p:Personne {nom:{1}}) RETURN p";
		try (Connection con = DriverManager.getConnection(JDBC_BOLT_URL, USER, PASSWORD)) {
			
			PreparedStatement pstmt = con.prepareStatement(readQuery);
			pstmt.setString(1, "Patricia");
			
			try (ResultSet rs = pstmt.executeQuery()) {
	            while (rs.next()) { 
	            	Map<String,Object> noeud = (Map<String,Object>) rs.getObject("p");
	            	Personne personne = MAPPER.convertValue(noeud, Personne.class);
	                System.out.println("Personne objet : "+personne.getNom()+" labels :"+personne.get_labels());
	            }
	        }
			catch (SQLException err) {			
				err.printStackTrace();
				Assert.fail(err.getMessage());
			}

			
			
		} catch (SQLException e) {			
			e.printStackTrace();
			
			Assert.fail(e.getMessage());
		}

		
		
	}
	
	/**
	 * Objet Personne
	 * @author sroussy
	 *
	 */
	public static class Personne{
		
		private Long _id;
		private List<String> _labels;
		private String nom;
		public Long get_id() {
			return _id;
		}
		public void set_id(Long _id) {
			this._id = _id;
		}
		public List<String> get_labels() {
			return _labels;
		}
		public void set_labels(List<String> _labels) {
			this._labels = _labels;
		}
		public String getNom() {
			return nom;
		}
		public void setNom(String nom) {
			this.nom = nom;
		}
	}
	
	
}
